<?php

namespace App\Controller;

use App\Exception\ItemException as ItemException;
use App\Exception\IntegrationException as IntegrationException;
use App\Exception\ParamsException as ParamsException;
use App\Exception\SqlException as SqlException;
use App\Exception\CancelException as CancelException;
use App\Exception\NetworkException as NetworkException;
use App\Exception\GeneralErrorException as GeneralErrorException;
use App\Exception\MenuException as MenuException;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

class WebService {

    const FAILOVER_EMAIL = "delivery@teknisa.com";

    private $connection;
    private $branchService;
    private $consumerService;
    private $dispatchService;
    private $integrationService;
    private $orderService;
    private $paymentService;
    private $productService;
    private $util;
    
    private $mailList;
    
    const API_ANT_BASE_URL = "https://api-parceiros.anota.ai/partnerauth/";
    const GEOCODE_URL      = "https://maps.google.com/maps/api/geocode/json?key=AIzaSyBfblcHNG4xb8lcwzD5sUvQ-dXIuY3QbNs=";

    public function __construct(\Doctrine\DBAL\Connection $connection,
                                \App\Service\BranchService $branchService,
                                \App\Service\ConsumerService $consumerService,
                                \App\Service\DispatchService $dispatchService,
                                \App\Service\IntegrationService $integrationService,
                                \App\Service\OrderService $orderService, 
                                \App\Service\PaymentService $paymentService,
                                \App\Service\ProductService $productService,
                                \App\Util\Util $util)
    {

        $this->connection         = $connection;
        $this->branchService      = $branchService;
        $this->consumerService    = $consumerService;
        $this->dispatchService    = $dispatchService;
        $this->integrationService = $integrationService;
        $this->orderService       = $orderService;
        $this->paymentService     = $paymentService;
        $this->productService     = $productService;
        $this->util               = $util;
    }

    public function showVersion(Request $request, Response $response)
    {
        $sql        = "SELECT CDFILIAL FROM FILIAL";
        $result     = $this->connection->fetchAll($sql);

        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'branchs'     => count($result)
        ]);
    }
    
    private function getVersionInfo()
    {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    /**
    * Função de entrada dos pedidos provenientes da Anota Aí, os pedidos chegam pelo WebHook respectivo
    *
    * @param Request $request
    * @param Response $response
    * @throws ParamsException Exceção de parâmetros inválidos ou faltantes.
    * @throws ItemException Exceção de parâmetros inválidos nos itens do pedido.
    * @throws SqlException Exceção de erros SQL.
    * @throws Exception Exceções gerais.
    * @return void
    */
    public function runWebService(Request $request, Response $response)
    {
        $params = $request->getParsedBody();

        // $this->debugLog($params);

        if ($params["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E") {
            $response->withStatus(400);
            return 0;
        }
            
        $errorMessage = "";
        $rawOrder     = "";
        $writeEmail   = true;
        
        $this->mailList = $this->util->getMailList();
        
        $configuration = $this->integrationService->getConfiguration();
        if(empty($configuration)) {
            $response->withStatus(200);
            return 0;
        }

        $ufTimezone      = (isset($configuration['IDUFTIMEZONE']) && !empty($configuration['IDUFTIMEZONE'])) ? $configuration['IDUFTIMEZONE'] : "SP";
        $trackingEnabled = (isset($configuration['IDRASTREIAENTREGA']) && ($configuration['IDRASTREIAENTREGA'] == "S"));
        date_default_timezone_set($this->util->getCurrentTimezone(strtoupper($ufTimezone)));
        
        if($configuration['IDHABINTE'] == 'N') 
            return $response->withStatus(200);
        
        $this->connection->beginTransaction();

        try {
            $order          = $params["anotaAiData"];
            $debug          = isset($params['debug']) ? $params['debug'] : false;
            $merchantId     = $order["merchant"]["id"];
            $orderId        = $order["id"];
            $shortId        = $order["shortReference"];
            
            // Filtro de pedidos com origem diferente de Anota Ai
            if($order["salesChannel"] !== "anotaai") {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). ": [ Pedido Rejeitado  ] #" . $shortId . " foi ignorado, pois não possui origem Anota Ai." . PHP_EOL);
                return $response->withJson([
                    'success' => false,
                    'message' => 'Pedido ignorado pois não possui origem Anota Ai',
                    'result'  => [
                        'orderId' => $params['anotaAiData']['id']
                    ]
                ]);
            }

            $rawOrder       = $order;
            $deliveryParams = $this->orderService->getDeliveryMethodParams($merchantId);
    
            if(!empty($deliveryParams['IDTOKEN'])) {
                $this->util->setAccessToken($deliveryParams['IDTOKEN']);
            } else {
                throw new GeneralErrorException("Token geral não encontrado, associado ao id ".$merchantId." do estabelecimento.");
            }    

            $deliveryParams['insertChange'] = ($deliveryParams['IDRETTXENTR'] == "N");

            $fileName = ((!empty($deliveryParams["CDFILIAL"])) && (!empty($shortId))) ? "{$deliveryParams['CDFILIAL']}-{$shortId}-{$this->util->getCurrentDateTime()}.json" : ("INDEFINIDO-" . $this->util->getCurrentDateTime()) . ".json";
            
            $this->util->writeLog('order', $fileName, $order);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). ": [  Pedido Recebido  ] #" . $shortId . " - Filial: " . $deliveryParams["CDFILIAL"] . PHP_EOL);
            
            $order["cdFilial"] = $deliveryParams["CDFILIAL"];
            
            $integrationDetails = $this->processOrder($order);
            
            $branchId = $integrationDetails["CDFILIAL"];
            $sellId  = $integrationDetails["NRVENDAREST"];
            $this->orderService->validateOrder($branchId, $sellId);
            $this->util->writeLog('order_processed', $fileName, $order);

            if(!$debug) {
              $acceptOrder = $this->acceptOrder($orderId, $deliveryParams['IDTOKEN']);
              $this->util->writeLog('accepted', $fileName, $acceptOrder);
            }
            $this->connection->commit();
            
            //rotina de dispatch ficará em outro módulo
            if($trackingEnabled) {
                $orderType = !empty($order['type']) ? $order['type'] : "";
                $this->dispatchService->registerPendingDispatchOrder($integrationDetails['CDFILIAL'], $integrationDetails['NRCOMANDA'], $integrationDetails['NRCOMANDAEXT'], $order['id'], $orderType, $merchantId);
            }
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). ": [ Pedido Integrado  ] #" . $shortId . " foi integrado com sucesso." . PHP_EOL);
            
        } catch(ItemException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [Erro Parametrizacao] -> {$e->getMessage()}" . PHP_EOL);
            $this->util->writeLog("order_error", $fileName, $errorMessage);
            
            $this->connection->rollback();
            if(!$debug){
                $cdFilial       = $this->getBranch($merchantId);
                $responseReject = $this->rejectOrder($orderId, $deliveryParams['IDTOKEN'], $shortId, $cdFilial);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(ParamsException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("order_error", $fileName, $errorMessage);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [Erro Parametrizacao] #" . $shortId . " -> ". $e->getMessage() . PHP_EOL);
            
            $this->connection->rollback();
            
            if(!$debug){
                $cdFilial       = $this->getBranch($merchantId);
                $responseReject = $this->rejectOrder($orderId, $deliveryParams['IDTOKEN'], $shortId, $cdFilial);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            } 
            
            if($debug){
                return $response->withJson([
                    'success' => false,
                    'message' => 'O pedido foi rejeitado como esperado',
                    'result'  => [
                        'orderId' => $params['anotaAiData']['id']
                    ]
                ]);
            }
        } catch(SqlException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("order_error", $fileName, $errorMessage);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [     Erro  SQL     ] #" . $shortId . " -> ". $e->getMessage() . PHP_EOL);
            
            $this->connection->rollback();
            
            if(!$debug){
                $cdFilial       = $this->getBranch($merchantId);
                $responseReject = $this->rejectOrder($orderId, $deliveryParams['IDTOKEN'], $shortId, $cdFilial);    
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(NetworkException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("network", $fileName, $errorMessage);
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [   Erro de Rede    ] -> {$e->getMessage()}" . PHP_EOL);
            
            $this->connection->rollback();
            
            if(!$debug){
                $cdFilial       = $this->getBranch($merchantId);
                $responseReject = $this->rejectOrder($orderId, $deliveryParams['IDTOKEN'], $shortId, $cdFilial);
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(IntegrationException $e) {
            $writeEmail = false;
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [  Erro Integracao  ] -> {$e->getMessage()}" . PHP_EOL);
            $this->util->writeLog("order_error", $fileName, $errorMessage);
            
            $this->connection->rollback();
            
            if(!$debug){
                $cdFilial       = $this->getBranch($merchantId);
                $responseReject = $this->rejectOrder($orderId, $deliveryParams['IDTOKEN'], $shortId, $cdFilial);
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } catch(CancelException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [ Erro Cancelamento ] -> {$e->getMessage()}" . PHP_EOL);
            $this->util->writeLog("cancelled_error", $fileName, $errorMessage);
            
            $this->connection->rollback();
        } catch(GeneralErrorException $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [   Erro Generico   ] -> {$e->getMessage()}" . PHP_EOL);
            $this->util->writeLog("error", $fileName, $errorMessage);
            
            $this->connection->rollback();
            
            if(!$debug){
                $cdFilial       = $this->getBranch($merchantId);
                $responseReject = $this->rejectOrder($orderId, $deliveryParams['IDTOKEN'], $shortId, $cdFilial); 
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
            
        } catch(\Exception $e) {
            $errorMessage = date("d-m-Y H:i:s") . " - " . $e->getMessage();
            if(empty($fileName)) $fileName = "INDEFINIDO-{$this->util->getCurrentDateTime()}.json";
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [   Erro Generico   ] -> {$e->getMessage()}" . PHP_EOL);
            $this->util->writeLog("error", $fileName, $errorMessage);
            
            $this->connection->rollback();

            if(!$debug){
                $cdFilial       = $this->getBranch($merchantId);
                $responseReject = $this->rejectOrder($orderId, $deliveryParams['IDTOKEN'], $shortId, $cdFilial); 
                $this->util->writeLog('rejected', $fileName, $responseReject);
            }
        } 
        if (!empty($errorMessage) && !$debug) {
            $cdFilial    = $this->getBranch($merchantId);
            $orderNumber = (isset($shortId) ? 
                        ("$cdFilial-{$shortId}") : ("INDEFINIDO-" . date("H-i-s"))); 

            if($writeEmail) {
                global $CLIENT_NAME;
                $clientName = $CLIENT_NAME;

                $mailBody = file_get_contents(__DIR__ . "/../../templates/defaultMail.html");

                if($debug) $this->mailList = null; 

                $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], 'Integração Teknisa x Anota Aí', $mailBody, [
                    ":INTEGRACAO"    => 'Teknisa x Anota Aí',
                    ":NUMERO_PEDIDO" => $orderNumber,
                    ":NOME_CLIENTE"  => $clientName,
                    ":NOME_SERVIDOR" => "",
                    ":ERRO_MSG"      => $errorMessage,
                    ":JSON_PEDIDO"   => json_encode($rawOrder, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                    ":GERADO_EM"     => date("d/m/Y H:i:s")
                ]);
            }
        }

        return $response->withJson([
            'success' => true,
            'message' => '',
            'result'  => [
                'orderId' => $params['anotaAiData']['id']
            ]
        ]);
    }

    private function processOrder($order)
    {
        $generalParams = $this->integrationService->generalParameters($order['cdFilial']);
        
        // TOGO - Insert branch address to order
        if($order["type"] == "TAKE") {
            $branchInfoAddress                           = $this->branchService->getBranchAddress($order['cdFilial']);
            $order["deliveryAddress"]["state"]           = $branchInfoAddress["SGESTADO"];
            $order["deliveryAddress"]["city"]            = $branchInfoAddress["CDMUNICIPIO"];
            $order["deliveryAddress"]["neighborhood"]    = $branchInfoAddress["NMBAIRFILI"];
            $order["deliveryAddress"]["streetNumber"]    = preg_replace("/[^0-9]/", "", $branchInfoAddress["DSENDEFILI"]);
            $order["deliveryAddress"]["streetName"]      = str_replace($order["deliveryAddress"]["streetNumber"], "", $branchInfoAddress["DSENDEFILI"]);
            $order["deliveryAddress"]["postalCode"]      = $branchInfoAddress["NRCEPFILI"];
            $order["deliveryAddress"]["complement"]      = "";
            $order["deliveryAddress"]["reference"]       = "";
            $order["deliveryAddress"]["lat"]             = "";
            $order["deliveryAddress"]["lng"]             = "";
        }

        $consumer = $this->consumerService->processConsumer($generalParams, $order);
        return $this->integrationService->openOrderBill($generalParams, $order, $consumer);
    }

    private function getBranch($token)
    {
        $branch = $this->orderService->getBranch($token);

        if(empty($branch)) 
            throw new ParamsException("Nao foi encontrado filial associada ao token: $token.");
        
        return $branch;
    }


    private function acceptOrder($orderId, $idToken)
    {
        $url         = self::API_ANT_BASE_URL."order/accept/$orderId";
        $body        = json_encode([]);

        $response = $this->util->executeWebService($url, "POST", $body, $idToken);

        return json_decode($response, true);
    }

    private function rejectOrder($orderId, $idToken, $shortId, $cdFilial)
    {
        $cancellationId = 922;
        $url          = self::API_ANT_BASE_URL."order/cancel/$orderId";
        $body         = json_encode(["justification"=>"NAO FOI POSSIVEL REALIZAR INTEGRACAO"]);
        
        $response = $this->util->executeWebService($url, "POST", $body, $idToken);
        $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). ": [  Pedido Recusado  ] #" . $shortId . " - Filial: " . $cdFilial . PHP_EOL);
        return json_decode($response, true);
    }
 
    public function cancelOrder(Request $request, Response $response)
    {
        try {
            $body     = $request->getParsedBody();
            $orderId  = $body['anotaAiData']['id'];
            $cdFilial = $body['anotaAiData']['merchant']['id'];
            $debug    = $body['debug'] ? $body['debug'] : false;
            
            if(!$debug){
                $this->orderService->cancelOrder($orderId);
                $order = $this->getOrderDetailsById($orderId);
                
                $fileName = ((!empty($order["CDFILIAL"])) && (!empty($orderId))) ? "{$order['CDFILIAL']}-{$orderId}-{$this->util->getCurrentDateTime()}.json" : ("INDEFINIDO-" . $this->util->getCurrentDateTime()) . ".json";
    
                $this->util->writeLog('cancelled', $fileName, $body);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). ": [ Pedido Cancelado  ] #" . $orderId . " - Filial: " . $cdFilial . PHP_EOL);
            }
        } catch(CancelException $e) {
            $errorMessage = $e->getMessage();
        } catch(\Exception $e) {
            $errorMessage = $e->getMessage();
        }
        if (!empty($errorMessage)) {
            $this->util->writeLog('cancelled_error', date("d-m-Y H:i:s") . '.txt', $errorMessage);
        }

        return $response->withStatus(200);
    }
    
    public function getOrderDetailsById($orderId)
    {
        return $this->orderService->getOrderDetailsById($orderId);
    }

    public function dispatch(Request $request, Response $response)
    {
        $req = $request->getParsedBody();
        
        $pendingOrdersFile = $this->util->getPendingOrdersFile();
        $this->filterDispatchFile($pendingOrdersFile);
        $this->processPendingDispatchOrders($pendingOrdersFile);
        $this->processPendingDeliveredOrders($pendingOrdersFile);
    }
    
    private function filterDispatchFile($pendingOrdersFile)
    {
        $pendingDispatchFile = file_get_contents($pendingOrdersFile);   
        $pendingDispatchData = json_decode($pendingDispatchFile, true);

        if(empty($pendingDispatchData)) {
            return 0;
        }

        $activeList = [];
        foreach ($pendingDispatchData as $key => $currentOrder) {
            
            $diff = (time() - $currentOrder['createdAt']);
            if($diff > 28800) { //8h
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [  Pedido Expirado  ] #" . $shortId . " expirado (não foi expedido dentro do tempo limite de 8h)." . PHP_EOL);
                continue;
            }

            $activeList[$key] = $currentOrder;
        }
        
        file_put_contents($pendingOrdersFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    private function processPendingDispatchOrders($pendingOrdersFile)
    {
        $url                 = self::API_ANT_BASE_URL . "order/";
        $body                = json_encode([]);
        $pendingDispatchFile = file_get_contents($pendingOrdersFile);   
        $pendingDispatchData = json_decode($pendingDispatchFile, true);

        if(empty($pendingDispatchData)) {
            return 0;
        }

        $cdFilialArr     = [];
        $nrComandaArr    = [];
        $nrComandaExtArr = [];
        $merchantId      = [];

        foreach($pendingDispatchData as $pendingDispatchOrder) {
            
            if(!empty($pendingDispatchOrder['dispatchedAt']) || $pendingDispatchOrder['delivered']) {
                continue;
            }
            
            if(!isset($pendingDispatchOrder['cdFilial']) || empty($pendingDispatchOrder['cdFilial'])) {
                continue;                
            }            
            
            if(!isset($pendingDispatchOrder['nrComanda']) || empty($pendingDispatchOrder['nrComanda'])) {
                continue;                
            }            
            
            if(!isset($pendingDispatchOrder['nrComandaExt']) || empty($pendingDispatchOrder['nrComandaExt'])) {
                continue;                
            }
            
            if(!isset($pendingDispatchOrder['merchantId']) || empty($pendingDispatchOrder['merchantId'])) {
                continue;                
            }            

            array_push($cdFilialArr,     $pendingDispatchOrder['cdFilial']);
            array_push($nrComandaArr,    $pendingDispatchOrder['nrComanda']);
            array_push($nrComandaExtArr, $pendingDispatchOrder['nrComandaExt']);
            array_push($merchantId,      $pendingDispatchOrder['merchantId']);
        }
        
        if(empty($cdFilialArr))
            return 0;

        if(empty($nrComandaArr))
            return 0;

        if(empty($nrComandaExtArr))
            return 0;
            
        if(empty($merchantId))
            return 0;
        
        $orders = $this->dispatchService->getPendingDispatchOrders($cdFilialArr, $nrComandaArr, $nrComandaExtArr, $merchantId);
        
        if(empty($orders))
            return 0;
        
        foreach($orders as $order) {
            $key = "{$order['CDFILIAL']}|{$order['NRCOMANDA']}|{$order['NRCOMANDAEXT']}";
            
            if($order['IDSTCOMANDA'] == '4') {
                if(isset($pendingDispatchData[$key])) {
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s"). ": [ Pedido Cancelado  ] #" . $pendingDispatchData[$key]['nrComandaExt'] . " removido da fila de controle de entregador pois foi cancelado." . PHP_EOL);
                    $cdFilial    = $this->getBranch($merchantId);
                    $this->rejectOrder($pendingDispatchData[$key]['reference'], $order['IDTOKEN']);
                    $pendingDispatchData[$key]['canceled'] = true;
                }
            }
            
            if($order['IDSTCOMANDA'] == '5' || $order['IDSTCOMANDA'] == 'X') {
                if(isset($pendingDispatchData[$key])) {
                    $reference = $pendingDispatchData[$key]['reference'];
                    $orderType = $pendingDispatchData[$key]['orderType'];
                    $shortId   = $pendingDispatchData[$key]['nrComandaExt']; //TESTAR PARA VER SE VOU PRECISAR DESSE SHORTID OU NÃO
                    
                    if ($orderType == "DELIVERY") {
                        $this->util->executeWebService($url . "ready/" . $reference, "POST", $body, $order['IDTOKEN']);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [  Pedido Delivery  ] #" . $shortId . " saiu para entrega." . PHP_EOL);
                        $pendingDispatchData[$key]['dispatchedAt'] = time();    
                    } else if ($orderType == "TAKE") { 
                        $this->util->executeWebService($url . "ready/" . $reference, "POST", $body, $order['IDTOKEN']);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [  Pedido Retirada  ] #" . $shortId . " pronto para retirada." . PHP_EOL);
                        $pendingDispatchData[$key]['dispatchedAt'] = time();
                    } else if ($orderType == "LOCAL") { 
                        $this->util->executeWebService($url . "ready/" . $reference, "POST", $body, $order['IDTOKEN']);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [    Pedido Mesa    ] #" . $shortId . " será entregue na sua mesa." . PHP_EOL);
                        $pendingDispatchData[$key]['dispatchedAt'] = time();
                    } else {
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $shortId . " foi removido da fila de entrega." . PHP_EOL);
                        $pendingDispatchData[$key]['canceled'] = true;
                    }
                }
            }
        }
        
        $activeList = [];
        
        foreach ($pendingDispatchData as $key => $currentOrder) {
            if($currentOrder['canceled']) {
                continue;
            }
            $activeList[$key] = $currentOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }

    private function processPendingDeliveredOrders($pendingOrdersFile)
    {
        $url                  = self::API_ANT_BASE_URL . "order/";
        $body                 = json_encode([]);
        $pendingDeliveredFile = file_get_contents($pendingOrdersFile);   
        $pendingDeliveredData = json_decode($pendingDeliveredFile, true);

        if(empty($pendingDeliveredData))
            return 0;
            
        

        foreach($pendingDeliveredData as $key => $pendingDeliveredOrder) {
            
            $idToken = $this->dispatchService->getFilialToken($pendingDeliveredOrder['cdFilial'], $pendingDeliveredOrder['merchantId']);
            
            if(empty($pendingDeliveredOrder['dispatchedAt'])) {
                continue;
            }

            $diff = (time() - $pendingDeliveredOrder['dispatchedAt']);
            if($diff >= 3600) {//3600
                $reference = $pendingDeliveredOrder['reference'];
                $this->util->executeWebService($url . "/finalize/" . $reference, "POST", $body, $idToken);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [ Pedido Finalizado ] #" . $pendingDeliveredOrder['nrComandaExt'] . " finalizado." . PHP_EOL);
                $pendingDeliveredData[$key]['delivered'] = true;
            }
        }
    
        $newPendingList = [];
        
        foreach($pendingDeliveredData as $key => $pendingDeliveredOrder) {
            if($pendingDeliveredOrder['delivered']) {
                continue;
            }
            $newPendingList[$key] = $pendingDeliveredOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($newPendingList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }
}